/* artist.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "artist.h"
#include "tableDataProxy.h"
#include "dataCell.h"
#include "database.h"
#include "artistParser.h"
#include "string_utils.h"

#include <ctype.h>
#include <algorithm>

string
get_formatted_artist_name(const TableDataProxy &artist) {
  assert(artist.get_table_id() == TI_artists);

  string last_name = artist[AFI_last_name].get_string();
  string first_name = artist[AFI_first_name].get_string();
  string name_suffix = artist[AFI_name_suffix].get_string();
  string group_prefix = artist[AFI_group_prefix].get_string();
  string group_name = artist[AFI_group_name].get_string();

  string individual_name;

  if (first_name.empty()) {
    individual_name = last_name;
  } else if (last_name.empty()) {
    individual_name = first_name;
  } else {
    individual_name = first_name + " " + last_name;
  }

  if (!name_suffix.empty()) {
    individual_name += " " + name_suffix;
  }

  if (!group_prefix.empty()) {
    group_name = group_prefix + " " + group_name;
  }

  if (individual_name.empty()) {
    return group_name;
  } else if (group_name.empty()) {
    return individual_name;
  } else {
    return individual_name + " and " + group_name;
  }
}

string
get_sorting_artist_name(const TableDataProxy &artist) {
  assert(artist.get_table_id() == TI_artists);

  string last_name = artist[AFI_last_name].get_string();
  string first_name = artist[AFI_first_name].get_string();
  string name_suffix = artist[AFI_name_suffix].get_string();
  string group_prefix = artist[AFI_group_prefix].get_string();
  string group_name = artist[AFI_group_name].get_string();

  string individual_name;

  if (first_name.empty()) {
    individual_name = last_name;
  } else if (last_name.empty()) {
    individual_name = first_name;
  } else {
    individual_name = last_name + ", " + first_name;
  }

  if (!name_suffix.empty()) {
    individual_name += " " + name_suffix;
  }

  if (individual_name.empty()) {
    if (!group_prefix.empty()) {
      group_name = group_name + ", " + group_prefix;
    }
    return group_name;
  } else if (group_name.empty()) {
    return individual_name;
  } else {
    if (!group_prefix.empty()) {
      group_name = group_prefix + " " + group_name;
    }
    return individual_name + " and " + group_name;
  }
}

/*
string
get_artist_db_sort_key(const TableDataProxy &artist) {
  assert(artist.get_table_id() == TI_artists);

  string last_name = artist[AFI_last_name].get_string();
  string first_name = artist[AFI_first_name].get_string();
  string group_name = artist[AFI_group_name].get_string();

  return make_upper(last_name + first_name + group_name);
}
*/
  



// Look in the database for an artist with the given name.  First we
// have to parse the name.  It might be a name of the form "Last,
// First", or it might be "First Last", or it might be a group name.
TableDataProxy
find_artist(string name) {
  TableDataProxy artist(TI_artists);

  name = trim_blanks(name);

  if (name.empty()) {
    // The empty string matches the empty artist.
    return artist;
  }

  ArtistParser p(name);

  artist[AFI_last_name] = DataCell(p._last_name);
  artist[AFI_first_name] = DataCell(p._first_name);
  artist[AFI_name_suffix] = DataCell(p._name_suffix, !p._name_suffix.empty());
  artist[AFI_group_prefix] = DataCell(p._group_prefix, !p._group_prefix.empty());
  artist[AFI_group_name] = DataCell(p._group_name);

  // Maybe this artist is already in the database.
  QueryResult qr = music_db->sql_command
    ("SELECT artist_id, "
     "last_name, first_name, name_suffix, "
     "group_prefix, group_name "
     "FROM artists"
     " WHERE last_name=" + artist[AFI_last_name] +
     " AND first_name=" + artist[AFI_first_name] +
     " AND group_name=" + artist[AFI_group_name] +
     " ORDER BY artist_id");  // In case there are multiple.

  if (qr.get_num_rows() == 0) {
    // Nope, not as we interpreted it.  But maybe if we reinterpreted
    // it specifically as an individual?
    ArtistParser p2(get_formatted_artist_name(artist),
		    ArtistParser::H_individual);
    TableDataProxy a2(TI_artists);

    a2[AFI_last_name] = DataCell(p2._last_name);
    a2[AFI_first_name] = DataCell(p2._first_name);
    a2[AFI_name_suffix] = DataCell(p2._name_suffix, !p2._name_suffix.empty());
    a2[AFI_group_prefix] = DataCell(p2._group_prefix, !p2._group_prefix.empty());
    a2[AFI_group_name] = DataCell(p2._group_name);

    if (a2[AFI_last_name] != artist[AFI_last_name] ||
	a2[AFI_first_name] != artist[AFI_first_name] ||
	a2[AFI_group_name] != artist[AFI_group_name]) {
      qr = music_db->sql_command
	("SELECT artist_id, "
	 "last_name, first_name, name_suffix, "
	 "group_prefix, group_name "
	 "FROM artists"
	 " WHERE last_name=" + a2[AFI_last_name] +
	 " AND first_name=" + a2[AFI_first_name] +
	 " AND group_name=" + a2[AFI_group_name] +
	 " ORDER BY artist_id");
    }
  }

  if (qr.get_num_rows() == 0) {
    // No, not as an individual either.  One last shot--maybe it's a
    // group?
    ArtistParser p2(get_formatted_artist_name(artist),
		    ArtistParser::H_group);
    TableDataProxy a2(TI_artists);

    a2[AFI_last_name] = DataCell(p2._last_name);
    a2[AFI_first_name] = DataCell(p2._first_name);
    a2[AFI_name_suffix] = DataCell(p2._name_suffix, !p2._name_suffix.empty());
    a2[AFI_group_prefix] = DataCell(p2._group_prefix, !p2._group_prefix.empty());
    a2[AFI_group_name] = DataCell(p2._group_name);

    if (a2[AFI_last_name] != artist[AFI_last_name] ||
	a2[AFI_first_name] != artist[AFI_first_name] ||
	a2[AFI_group_name] != artist[AFI_group_name]) {
      qr = music_db->sql_command
	("SELECT artist_id, "
	 "last_name, first_name, name_suffix, "
	 "group_prefix, group_name "
	 "FROM artists"
	 " WHERE last_name=" + a2[AFI_last_name] +
	 " AND first_name=" + a2[AFI_first_name] +
	 " AND group_name=" + a2[AFI_group_name] +
	 " ORDER BY artist_id");
    }
  }
    
  if (qr.get_num_rows() != 0) {
    // The artist does exist in the database!  Set up the table to
    // refer to it.
    artist.set_primary_key(qr.get_cell(0, 0).get_int(), false);
    artist.set_db_value(AFI_last_name, qr.get_cell(0, 1));
    artist.set_db_value(AFI_first_name, qr.get_cell(0, 2));
    artist.set_db_value(AFI_name_suffix, qr.get_cell(0, 3));
    artist.set_db_value(AFI_group_prefix, qr.get_cell(0, 4));
    artist.set_db_value(AFI_group_name, qr.get_cell(0, 5));
  }

  return artist;
}
